APPROXIMATION = "\u2248"
UP = "\u2191"
DOWN = "\u2193"
LEFT = "\u2190"
RIGHT = "\u2192"
HOME = "\u2196"
END = "\u2198"
PAGE_DOWN = "\u21A7"
PAGE_UP = "\u21A5"
RIGHT_TRIANGLE = "\u25B6"
BIG_RIGHT_TRIANGLE = "\uE0B0"
DOWN_TRIANGLE = "\u25BC"
DIAMOND = "\u25C8"
